#!/bin/bash

# set sane defaults
IFS=$' \t\n'
export LANG=C
export LC_NUMERIC=C
export LC_TIME=C

[ -n "$SSH_OPTS" ] || SSH_OPTS='-o IdentitiesOnly=yes -o BatchMode=yes'
[ -n "$SSH_OPTS_EXTRA" ] || SSH_OPTS_EXTRA=''
[ -n "$SSH_TIMEOUT" ] || SSH_TIMEOUT='10'

die()
{
  [ "$1" == '0' ] && printf '%s\n' "$2" || printf '%s\n' "$2" 1>&2
  exit "$1"
}

syntax()
{
  die "$1" 'Usage: op5 node rawctrl <NODE> <cmd|--test>'
}

nodeaddr()
{
  mon node show "$1" 2>/dev/null | grep ^ADDRESS= | cut -d= -f2
}

cmdrun()
{
  if [ -n "$addr" ]; then
    ssh -o ConnectTimeout="$SSH_TIMEOUT" \
      $SSH_OPTS $SSH_OPTS_EXTRA "$addr" "$cmd"
  else
    bash -c "$cmd"
  fi
}

cmdtest()
{
  local cmd msg out ret

  cmd="printf '%s\n' test-{a,b}{,-x}-{0,1} | cat"

  printf 'Testing (%s)... ' "$name"

  out="$(cmdrun "$name" "$addr" "$cmd" 2>&1)"
  ret="$?"

  fail() { die '1' $'failed.\n'"$(set | egrep '^(name|addr|cmd|ret|out)=')"; }

  [ "$ret" == '0' ] || fail
  [ "$out" == "$(printf '%s\n' test-{a,b}{,-x}-{0,1})" ] || fail

  die '0' 'ok'
}

[ "$1" == '--help' ] && syntax 0
[ "$1" == '-h' ] && syntax 0

[ -n "$2" ] || syntax 1
[ -n "$3" ] && syntax 1

name="$1"
cmd="$2"
addr="$(nodeaddr "$name")"

if [ "$name" == "$HOSTNAME" ]; then
  if [ -n "$addr" ]; then
    msg="ERROR: Given node name ($name) interpreted as localhost, but this is"
    die '1' "$msg also the name of a remote merlin node."
  fi
  addr=''
elif [ -z "$addr" ]; then
  die '1' "ERROR: Unable to find address of node ($name)."
fi

if [ "$cmd" == '--test' ]; then
  cmdtest "$addr"
else
  cmdrun "$addr" "$cmd"
fi
